
# do not make coreutils-single depend on /usr/bin/coreutils
%global __requires_exclude ^%{_bindir}/coreutils$

Summary:A set of basic GNU tools commonly used in shell scripts
Name:		coreutils
Version:	9.4
Release:	9%{?dist}
License:	GPLv3+
Url:		https://www.gnu.org/software/coreutils/
Source0:	https://ftp.gnu.org/gnu/%{name}/%{name}-%{version}.tar.xz
Source1:    	colorls.sh
Source2:	coreutils-provides.inc
Source3:	supported_utils

Patch0001:  skip-some-tests-if-selinux-is-disabled.patch
Patch0002:  0001-CVE-2024-0684-split-do-not-shrink-hold-buffer.patch
# CVE-2025-5278
# https://github.com/coreutils/coreutils/commit/8c9602e3a145e9596dc1a63c6ed67865814b6633
Patch0003:  0001-CVE-2025-5278-sort-fix-buffer-under-read-CWE-127.patch
Patch3000:  coreutils-9.4-systemd-coredump.patch 
Patch5000:  add-sm3sum-support.patch
Patch5001:  0001-fix-uname-processortype-error-for-loongarch64.patch

%ifnarch loongarch64
BuildRequires:	strace
%endif
BuildRequires:	attr, autoconf, automake, gcc, hostname, make, texinfo, libselinux-utils
BuildRequires:	gettext-devel, gmp-devel, libacl-devel, libattr-devel, libcap-devel, libselinux-devel, openssl-devel
# test-only dependencies
BuildRequires:	perl-interpreter, perl(FileHandle)
# needed by i18n test-cases
BuildRequires:	glibc-langpack-en

Requires: %{name}-common = %{version}-%{release}
Conflicts: coreutils-single

Provides: coreutils-full = %{version}-%{release}
%include %{SOURCE2}

%description
The GNU Core Utilities are the basic file, shell and text manipulation utilities of the GNU operating system. 
These are the core utilities which are expected to exist on every operating system.

%package single
Summary:  coreutils multicall binary
Suggests: coreutils-common
Provides: coreutils = %{version}-%{release}
Provides: coreutils%{?_isa} = %{version}-%{release}
%include %{SOURCE2}
# To avoid clobbering installs
Conflicts: coreutils
RemovePathPostfixes: .single
%description single
These are the GNU core utilities, packaged as a single multicall binary.

%package common
# yum obsoleting rules explained at:
# https://bugzilla.redhat.com/show_bug.cgi?id=1107973#c7
Obsoletes: %{name} < 8.24-100
Requires(preun): /sbin/install-info
Requires(post): /sbin/install-info
Summary:  coreutils common optional components
%description common
Optional though recommended components,
including documentation and translations.

%prep
%autosetup -N

sed src/dircolors.hin \
        -e 's| 00;36$| 01;36|' \
        > DIR_COLORS

sed src/dircolors.hin \
        -e 's| 01;31$| 00;31|' \
        -e 's| 01;35$| 00;35|' \
        > DIR_COLORS.lightbgcolor

%autopatch -p1
find tests -name '*.sh' -perm 0644 -print -exec chmod 0755 '{}' '+'
autoreconf -fiv
%ifarch loongarch64
sed -i "/inotify-dir-recreate.sh/d" tests/local.mk
%endif

%build
export CFLAGS="$RPM_OPT_FLAGS -fno-strict-aliasing -fpic"
export ac_cv_func_lchmod="no"
export optflags="%{optflags} -D_GNU_SOURCE=1"

for type in separate single; do 
  mkdir -p $type && \
  (cd $type && ln -s ../configure || exit $?
  if test $type = 'single'; then
    config_single='--enable-single-binary'
    config_single="$config_single --without-openssl"  # smaller/slower sha*sum
    config_single="$config_single --without-gmp"      # expr/factor machine ints
  else
    config_single='--with-openssl'  # faster sha*sum
  fi
  export FORCE_UNSAFE_CONFIGURE=1 
  %configure $config_single\
             --cache-file=../config.cache \
             --enable-install-program=arch \
             --enable-no-install-program=kill,uptime \
             --with-tty-group \
             DEFAULT_POSIX2_VERSION=200112 alternative=199209 || :
  %make_build all V=1

  ln -fv ../lib/parse-datetime.{c,y} .
  )
done
# Get the list of supported utilities
cp %SOURCE3 .

%install
for type in separate single; do
  install=install
  if test $type = 'single'; then
    subdir=%{_libexecdir}/%{name}; install=install-exec
  fi
  (cd $type && make DESTDIR=$RPM_BUILD_ROOT/$subdir $install)

  # chroot was in /usr/sbin :
  mkdir -p $RPM_BUILD_ROOT/$subdir/{%{_bindir},%{_sbindir}}
  mv $RPM_BUILD_ROOT/$subdir/{%_bindir,%_sbindir}/chroot

  # Move multicall variants to *.single.
  # RemovePathPostfixes will strip ".single".
  if test $type = 'single'; then
    for dir in %{_bindir} %{_sbindir} %{_libexecdir}/%{name}; do
      for bin in $RPM_BUILD_ROOT/%{_libexecdir}/%{name}/$dir/*; do
        basebin=$(basename $bin)
        mv $bin $RPM_BUILD_ROOT/$dir/$basebin.single
      done
    done
  fi
done

%find_lang %name
grep LC_TIME %name.lang | cut -d'/' -f1-6 | sed -e 's/) /) %%dir /g' >>%name.lang

mkdir -p %buildroot%{_sysconfdir}/profile.d
install -p -c -m644 DIR_COLORS{,.lightbgcolor} %buildroot%{_sysconfdir}
install -p -c -m644 %{SOURCE1} %buildroot%{_sysconfdir}/profile.d/colorls.sh

rm -f $RPM_BUILD_ROOT%{_infodir}/dir

%check
pushd separate
make check VERBOSE=yes
popd

%preun common
if [ $1 = 0 ]; then
  if [ -f %{_infodir}/%{name}.info.gz ]; then
    /sbin/install-info --delete %{_infodir}/%{name}.info.gz %{_infodir}/dir || :
  fi
fi

%post common
if [ -f %{_infodir}/%{name}.info.gz ]; then
  /sbin/install-info %{_infodir}/%{name}.info.gz %{_infodir}/dir || :
fi

%files -f supported_utils
%exclude %{_bindir}/*.single
%dir %{_libexecdir}/coreutils
%{_libexecdir}/coreutils/*.so

%files single
# coreutils-single does not strongly depend on coreutils-common,
# So COPYRIGHT must be retained
%license COPYING
%{_bindir}/*.single
%{_sbindir}/chroot.single
%dir %{_libexecdir}/coreutils
%{_libexecdir}/coreutils/*.so.single

%files common -f %{name}.lang
%license COPYING
%doc ABOUT-NLS NEWS README THANKS TODO
%config(noreplace) %{_sysconfdir}/DIR_COLORS*
%config(noreplace) %{_sysconfdir}/profile.d/*
%{_infodir}/coreutils*
%{_mandir}/man*/*

%changelog
* Thu May 29 2025 Chunsheng Luo <luffyluo@tencent.com> - 9.4-9
- sort: fix buffer under-read (CWE-127)(CVE-2025-5278)

* Fri Feb 21 2025 Chunsheng Luo <luffyluo@tencent.com> - 9.4-8
- Fix useless code and code errors

* Tue Feb 18 2025 Chunsheng Luo <luffyluo@tencent.com> - 9.4-7
- Provide coreutils-single to more easily support smaller installations
- Split out coreutils-common to provide common documentation

* Thu Sep 26 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.4-6
- Rebuilt for clarifying the packages requirement in BaseOS and AppStream

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.4-5
- Rebuilt for loongarch release

* Tue Apr 2 2024 luffyluo <luffyluo@tencent.com> - 9.4-4
- Fix error changelog format

* Mon Mar 11 2024 Wenlong Zhang <zhangwenlong@loongson.cn> - 9.4-3
- add loongarch64 support for coreutils

* Mon Jan 29 2024 luffyluo <luffyluo@tencent.com> - 9.4-2
- Fix CVE-2024-0684

* Thu Jan 11 2024 Upgrade Robot <upbot@opencloudos.org> - 9.4-1
- Upgrade to version 9.4

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.3-2
- Rebuilt for OpenCloudOS Stream 23.09

* Wed Jul 26 2023 Miaojun Dong <zoedong@tencent.com> - 9.3-1
- Bump version to 9.3

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.1-5
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 9.1-4
- Rebuilt for OpenCloudOS Stream 23

* Thu Feb 16 2023 Miaojun Dong <zoedong@tencent.com> - 9.1-3
- add support for sm3

* Wed Dec 7 2022 Zhao Zhen <jeremiazhao@tencent.com> - 9.1-2
- added some added bash configures

* Thu Apr 28 2022 Songqiao Tao <joeytao@tencent.com> - 9.1-1
- initial build
